"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AptosClient = exports.raiseForStatus = exports.RequestError = void 0;
const Accounts_1 = require("./api/Accounts");
const Events_1 = require("./api/Events");
const Transactions_1 = require("./api/Transactions");
const http_client_1 = require("./api/http-client");
const hex_string_1 = require("./hex_string");
const util_1 = require("./util");
const Tables_1 = require("./api/Tables");
const transaction_builder_1 = require("./transaction_builder");
class RequestError extends Error {
    constructor(message, response, requestBody) {
        const data = JSON.stringify(response.data);
        const hostAndPath = [response.request?.host, response.request?.path].filter((e) => !!e).join('');
        super(`${message} - ${data}${hostAndPath ? ` @ ${hostAndPath}` : ''}${requestBody ? ` : ${requestBody}` : ''}`);
        this.response = response;
        this.requestBody = requestBody;
        Object.setPrototypeOf(this, new.target.prototype); // restore prototype chain
    }
}
exports.RequestError = RequestError;
function raiseForStatus(expectedStatus, response, requestContent) {
    if (response.status !== expectedStatus) {
        if (requestContent) {
            throw new RequestError(response.statusText, response, JSON.stringify(requestContent));
        }
        throw new RequestError(response.statusText, response);
    }
}
exports.raiseForStatus = raiseForStatus;
class AptosClient {
    constructor(nodeUrl, config) {
        this.nodeUrl = nodeUrl;
        // `withCredentials` ensures cookie handling
        this.client = new http_client_1.HttpClient({
            withCredentials: false,
            baseURL: nodeUrl,
            validateStatus: () => true,
            ...(config || {}),
        });
        // Initialize routes
        this.accounts = new Accounts_1.Accounts(this.client);
        this.tables = new Tables_1.Tables(this.client);
        this.events = new Events_1.Events(this.client);
        this.transactions = new Transactions_1.Transactions(this.client);
    }
    /** Returns the sequence number and authentication key for an account */
    async getAccount(accountAddress) {
        const response = await this.accounts.getAccount(hex_string_1.HexString.ensure(accountAddress).hex());
        raiseForStatus(200, response);
        return response.data;
    }
    /** Returns transactions sent by the account */
    async getAccountTransactions(accountAddress, query) {
        const response = await this.accounts.getAccountTransactions(hex_string_1.HexString.ensure(accountAddress).hex(), query);
        raiseForStatus(200, response);
        return response.data;
    }
    /** Returns all modules associated with the account */
    async getAccountModules(accountAddress, query) {
        const response = await this.accounts.getAccountModules(hex_string_1.HexString.ensure(accountAddress).hex(), query);
        raiseForStatus(200, response);
        return response.data;
    }
    /** Returns the module identified by address and module name */
    async getAccountModule(accountAddress, moduleName, query) {
        const response = await this.accounts.getAccountModule(hex_string_1.HexString.ensure(accountAddress).hex(), moduleName, query);
        raiseForStatus(200, response);
        return response.data;
    }
    /** Returns all resources associated with the account */
    async getAccountResources(accountAddress, query) {
        const response = await this.accounts.getAccountResources(hex_string_1.HexString.ensure(accountAddress).hex(), query);
        raiseForStatus(200, response);
        return response.data;
    }
    /** Returns the resource by the address and resource type */
    async getAccountResource(accountAddress, resourceType, query) {
        const response = await this.accounts.getAccountResource(hex_string_1.HexString.ensure(accountAddress).hex(), resourceType, query);
        raiseForStatus(200, response);
        return response.data;
    }
    /** Generates a signed transaction that can be submitted to the chain for execution. */
    static async generateBCSTransaction(accountFrom, rawTxn) {
        const txnBuilder = new transaction_builder_1.TransactionBuilderEd25519((signingMessage) => {
            // @ts-ignore
            const sigHexStr = accountFrom.signBuffer(signingMessage);
            return sigHexStr.toUint8Array();
        }, accountFrom.pubKey().toUint8Array());
        const signedTxn = await txnBuilder.sign(rawTxn);
        return signedTxn;
    }
    /** Generates a transaction request that can be submitted to produce a raw transaction that
     * can be signed, which upon being signed can be submitted to the blockchain. */
    async generateTransaction(sender, payload, options) {
        const senderAddress = hex_string_1.HexString.ensure(sender);
        const account = await this.getAccount(senderAddress);
        return {
            sender: senderAddress.hex(),
            sequence_number: account.sequence_number,
            max_gas_amount: '1000',
            gas_unit_price: '1',
            gas_currency_code: 'XUS',
            // Unix timestamp, in seconds + 10 seconds
            expiration_timestamp_secs: (Math.floor(Date.now() / 1000) + 10).toString(),
            payload,
            ...(options || {}),
        };
    }
    /** Converts a transaction request by `generate_transaction` into it's binary hex BCS representation, ready for
     * signing and submitting.
     * Generally you may want to use `signTransaction`, as it takes care of this step + signing */
    async createSigningMessage(txnRequest) {
        const response = await this.transactions.createSigningMessage(txnRequest);
        raiseForStatus(200, response, txnRequest);
        const { message } = response.data;
        return message;
    }
    /** Converts a transaction request produced by `generate_transaction` into a properly signed
     * transaction, which can then be submitted to the blockchain. */
    async signTransaction(accountFrom, txnRequest) {
        const message = await this.createSigningMessage(txnRequest);
        const signatureHex = accountFrom.signHexString(message.substring(2));
        const transactionSignature = {
            type: 'ed25519_signature',
            public_key: accountFrom.pubKey().hex(),
            signature: signatureHex.hex(),
        };
        return { signature: transactionSignature, ...txnRequest };
    }
    async getEventsByEventKey(eventKey) {
        const response = await this.events.getEventsByEventKey(eventKey);
        raiseForStatus(200, response, `eventKey: ${eventKey}`);
        return response.data;
    }
    async getEventsByEventHandle(address, eventHandleStruct, fieldName, query) {
        const response = await this.accounts.getEventsByEventHandle(hex_string_1.HexString.ensure(address).hex(), eventHandleStruct, fieldName, query);
        raiseForStatus(200, response, { address, eventHandleStruct, fieldName });
        return response.data;
    }
    /** Submits a signed transaction to the transaction endpoint that takes JSON payload. */
    async submitTransaction(signedTxnRequest) {
        const response = await this.transactions.submitTransaction(signedTxnRequest);
        raiseForStatus(202, response, signedTxnRequest);
        return response.data;
    }
    /** Submits a signed transaction to the the endpoint that takes BCS payload. */
    async submitSignedBCSTransaction(signedTxn) {
        // Need to construct a customized post request for transactions in BCS payload
        const httpClient = this.transactions.http;
        const response = await httpClient.request({
            path: '/transactions',
            method: 'POST',
            body: signedTxn,
            // @ts-ignore
            type: 'application/x.aptos.signed_transaction+bcs',
            format: 'json',
        });
        raiseForStatus(202, response, signedTxn);
        return response.data;
    }
    async getTransactions(query) {
        const response = await this.transactions.getTransactions(query);
        raiseForStatus(200, response);
        return response.data;
    }
    async getTransaction(txnHashOrVersion) {
        const response = await this.transactions.getTransaction(txnHashOrVersion);
        raiseForStatus(200, response, { txnHashOrVersion });
        return response.data;
    }
    async transactionPending(txnHash) {
        const response = await this.transactions.getTransaction(txnHash);
        if (response.status === 404) {
            return true;
        }
        raiseForStatus(200, response, txnHash);
        return response.data.type === 'pending_transaction';
    }
    /** Waits up to 10 seconds for a transaction to move past pending state */
    async waitForTransaction(txnHash) {
        let count = 0;
        // eslint-disable-next-line no-await-in-loop
        while (await this.transactionPending(txnHash)) {
            // eslint-disable-next-line no-await-in-loop
            await (0, util_1.sleep)(1000);
            count += 1;
            if (count >= 10) {
                throw new Error(`Waiting for transaction ${txnHash} timed out!`);
            }
        }
    }
    async getLedgerInfo(params = {}) {
        const result = await this.client.request({
            path: '/',
            method: 'GET',
            format: 'json',
            ...params,
        });
        return result.data;
    }
    async getChainId(params = {}) {
        const result = await this.getLedgerInfo(params);
        return result.chain_id;
    }
    async getTableItem(handle, data, params) {
        const tableItem = await this.tables.getTableItem(handle, data, params);
        return tableItem;
    }
}
exports.AptosClient = AptosClient;
//# sourceMappingURL=aptos_client.js.map