"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TokenClient = void 0;
class TokenClient {
    constructor(aptosClient) {
        this.aptosClient = aptosClient;
    }
    async submitTransactionHelper(account, payload) {
        const txnRequest = await this.aptosClient.generateTransaction(account.address(), payload, {
            max_gas_amount: '4000',
        });
        const signedTxn = await this.aptosClient.signTransaction(account, txnRequest);
        const res = await this.aptosClient.submitTransaction(signedTxn);
        await this.aptosClient.waitForTransaction(res.hash);
        return Promise.resolve(res.hash);
    }
    // Creates a new collection within the specified account
    async createCollection(account, name, description, uri) {
        const payload = {
            type: 'script_function_payload',
            function: '0x1::Token::create_unlimited_collection_script',
            type_arguments: [],
            arguments: [
                Buffer.from(name).toString('hex'),
                Buffer.from(description).toString('hex'),
                Buffer.from(uri).toString('hex'),
            ],
        };
        const transactionHash = await this.submitTransactionHelper(account, payload);
        return transactionHash;
    }
    // Creates a new token within the specified account
    async createToken(account, collectionName, name, description, supply, uri) {
        const payload = {
            type: 'script_function_payload',
            function: '0x1::Token::create_unlimited_token_script',
            type_arguments: [],
            arguments: [
                Buffer.from(collectionName).toString('hex'),
                Buffer.from(name).toString('hex'),
                Buffer.from(description).toString('hex'),
                true,
                supply.toString(),
                Buffer.from(uri).toString('hex'),
            ],
        };
        const transactionHash = await this.submitTransactionHelper(account, payload);
        return transactionHash;
    }
    // Offer token to another account
    async offerToken(account, receiver, creator, collectionName, name, amount) {
        const payload = {
            type: 'script_function_payload',
            function: '0x1::TokenTransfers::offer_script',
            type_arguments: [],
            arguments: [
                receiver,
                creator,
                Buffer.from(collectionName).toString('hex'),
                Buffer.from(name).toString('hex'),
                amount.toString(),
            ],
        };
        const transactionHash = await this.submitTransactionHelper(account, payload);
        return transactionHash;
    }
    // Claim token
    async claimToken(account, sender, creator, collectionName, name) {
        const payload = {
            type: 'script_function_payload',
            function: '0x1::TokenTransfers::claim_script',
            type_arguments: [],
            arguments: [sender, creator, Buffer.from(collectionName).toString('hex'), Buffer.from(name).toString('hex')],
        };
        const transactionHash = await this.submitTransactionHelper(account, payload);
        return transactionHash;
    }
    // Cancel token
    async cancelTokenOffer(account, receiver, creator, collectionName, name) {
        const payload = {
            type: 'script_function_payload',
            function: '0x1::TokenTransfers::cancel_offer_script',
            type_arguments: [],
            arguments: [receiver, creator, Buffer.from(collectionName).toString('hex'), Buffer.from(name).toString('hex')],
        };
        const transactionHash = await this.submitTransactionHelper(account, payload);
        return transactionHash;
    }
    async getCollectionData(creator, collectionName) {
        const resources = await this.aptosClient.getAccountResources(creator);
        const accountResource = resources.find((r) => r.type === '0x1::Token::Collections');
        const { handle } = accountResource.data.collections;
        const getCollectionTableItemRequest = {
            key_type: '0x1::ASCII::String',
            value_type: '0x1::Token::Collection',
            key: collectionName,
        };
        // eslint-disable-next-line no-unused-vars
        const collectionTable = await this.aptosClient.getTableItem(handle, getCollectionTableItemRequest);
        return collectionTable;
    }
    // Retrieve the token's creation_num, which is useful for non-creator operations
    async getTokenData(creator, collectionName, tokenName) {
        const collection = await this.aptosClient.getAccountResource(creator, '0x1::Token::Collections');
        const { handle } = collection.data.token_data;
        const tokenId = {
            creator,
            collection: collectionName,
            name: tokenName,
        };
        const getTokenTableItemRequest = {
            key_type: '0x1::Token::TokenId',
            value_type: '0x1::Token::TokenData',
            key: tokenId,
        };
        const tableItem = await this.aptosClient.getTableItem(handle, getTokenTableItemRequest);
        return tableItem;
    }
    // Retrieve the token's creation_num, which is useful for non-creator operations
    async getTokenBalance(creator, collectionName, tokenName) {
        const tokenStore = await this.aptosClient.getAccountResource(creator, '0x1::Token::TokenStore');
        const { handle } = tokenStore.data.tokens;
        const tokenId = {
            creator,
            collection: collectionName,
            name: tokenName,
        };
        const getTokenTableItemRequest = {
            key_type: '0x1::Token::TokenId',
            value_type: '0x1::Token::Token',
            key: tokenId,
        };
        const tableItem = await this.aptosClient.getTableItem(handle, getTokenTableItemRequest);
        return tableItem;
    }
}
exports.TokenClient = TokenClient;
//# sourceMappingURL=token_client.js.map